//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Windows.Forms;


namespace MindFusion.Scheduling.WinForms.Samples.CS.DualView
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		/// <summary>
		/// Initializes a new instance of the MainForm class.
		/// </summary>
		public MainForm()
		{
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			// Synchronize both views
			_schedule.Schedule = _calendar.Schedule;

			// Select the current date
			_calendar.Selection.Set(DateTime.Today);

			_calendar.MonthSettings.DaySettings.HeaderSize = 0;

			// Listen for selection changes
			_calendar.Selection.Changed += new EventHandler(OnSelectionChanged);

			// Listen for item creation/deletion/modification
			_schedule.ItemCreated += new EventHandler<MindFusion.Scheduling.ItemEventArgs>(OnItemCreated);
			_schedule.ItemDeleted += new EventHandler<MindFusion.Scheduling.ItemEventArgs>(OnItemDeleted);
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void OnItemCreated(object sender, MindFusion.Scheduling.ItemEventArgs e)
		{
			// Check if the item date already has custom style
			foreach (DateStyle s in _calendar.DayStyles)
			{
				if (s.From.Date == e.Item.StartTime.Date)
					return;
			}

			// No custom style for this date -> add one
			DateStyle style = new DateStyle();
			style.From = e.Item.StartTime.Date;
			style.To = e.Item.StartTime.Date;
			style.Style.HeaderFont = new Font("Tahoma", 8, FontStyle.Bold);
			_calendar.DayStyles.Add(style);
			_calendar.Invalidate();
		}

		private void OnItemDeleted(object sender, MindFusion.Scheduling.ItemEventArgs e)
		{
			// Check if there are more items on this date.
			// If no - remove the custom style
			DateTime start = e.Item.StartTime.Date;
			DateTime end = e.Item.EndTime;
			end = new DateTime(end.Year, end.Month, end.Day, 23, 59, 59);
			if (_schedule.Schedule.GetAllItems(start, end).Count == 0)
			{
				foreach (DateStyle s in _calendar.DayStyles)
				{
					if (s.From.Date == e.Item.StartTime.Date)
					{
						_calendar.DayStyles.Remove(s);
						break;
					}
				}
			}
		}

		private void OnSelectionChanged(object sender, EventArgs e)
		{
			if (_calendar.Selection.IsEmpty)
			{
				_schedule.TimetableSettings.Dates.Clear();
				return;
			}

			_schedule.BeginInit();
			_schedule.TimetableSettings.Dates.Clear();
			_schedule.TimetableSettings.Dates.Add((DateTime)_calendar.Selection.Ranges[0]);
			_schedule.EndInit();
		}


		private string _appPath;
	}
}
